package com.file.view;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import androidx.annotation.NonNull;

import com.tencent.smtt.export.external.TbsCoreSettings;
import com.tencent.smtt.sdk.QbSdk;
import com.tencent.smtt.sdk.TbsDownloader;
import com.tencent.smtt.sdk.TbsListener;

import java.util.HashMap;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * FlutterFileViewPlugin
 */
public class FlutterFileViewPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  static final String TAG = FlutterFileViewPlugin.class.getName();

  private MethodChannel channel;

  // 状态值
  public int initX5Status = X5Status.INIT;

  // 尝试重新下载的次数
  private static final int maxDownCount = 10;
  private int nowDownCount = 0;

  public static final String channelName = "flutter_file_view.io.method";
  public static final String viewName = "plugins.file_local_view/view";
  private FlutterPluginBinding mFlutterPluginBinding;
  private Context mContext;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), channelName);
    channel.setMethodCallHandler(this);

    this.mFlutterPluginBinding = flutterPluginBinding;
    this.mContext = mFlutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case "getPlatformVersion":
        result.success("Android " + android.os.Build.VERSION.RELEASE);
        break;
      case "getX5Status":
        result.success(getX5Status());
        break;
      case "manualInitX5":
        manualInitX5();
        break;
      default:
        result.notImplemented();
        break;
    }
  }

  /**
   * 初始化 X5
   */
  public void initX5() {
    final Handler handler = new Handler(Looper.myLooper()) {
      @Override
      public void dispatchMessage(@NonNull Message msg) {
        super.dispatchMessage(msg);
        channel.invokeMethod("x5Status", getX5Status());
      }
    };

    handleMessage(handler);
    if (mContext == null) {
      return;
    }

    Log.e(TAG, "开始初始化X5");

    // 更改初始化状态
    initX5Status = X5Status.INIT_LOADING;
    handleMessage(handler);

    /// 不获取 AndroidID
    QbSdk.canGetAndroidId(false);
    /// 不获取 IMEI（设备识别码）
    QbSdk.canGetDeviceId(false);
    /// 不获取 IMSI（用户识别码）
    QbSdk.canGetSubscriberId(false);

    resetQbSdk();

    // 在调用TBS初始化、创建WebView之前进行如下配置，以开启优化方案
    HashMap<String, Object> map = new HashMap<>();
    map.put(TbsCoreSettings.TBS_SETTINGS_USE_SPEEDY_CLASSLOADER, true);
    map.put(TbsCoreSettings.TBS_SETTINGS_USE_DEXLOADER_SERVICE, true);
    QbSdk.initTbsSettings(map);

    QbSdk.setDownloadWithoutWifi(true);
    QbSdk.setTbsListener(new TbsListener() {
      @Override
      public void onDownloadFinish(int i) {
        initX5Status = i == 100 || i == 0 ? X5Status.DOWNLOAD_SUCCESS : X5Status.DOWNLOAD_FAIL;
        handleMessage(handler);
        
        Log.e(TAG, "TBS下载完成 - " + i + " - " + showStatus(i == 100 || i == 0));

        if (initX5Status == X5Status.DOWNLOAD_FAIL && nowDownCount <= maxDownCount) {
          // 下载失败后，尝试重新下载
          TbsDownloader.startDownload(mContext);
          nowDownCount++;
        }
      }

      @Override
      public void onInstallFinish(int i) {
        initX5Status = i == 200 ? X5Status.INSTALL_SUCCESS : X5Status.INSTALL_FAIL;
        handleMessage(handler);
        
        Log.e(TAG, "TBS安装完成 - " + i + " - " + showStatus(i == 200));
      }

      @Override
      public void onDownloadProgress(int i) {
        initX5Status = i > 0 && i <= 100 ? X5Status.DOWNLOAD_LOADING : X5Status.DOWNLOAD_FAIL;
        Log.e(TAG, "TBS下载进度:" + i);
      }
    });

    QbSdk.PreInitCallback onQbSdkPreInitCallback = new QbSdk.PreInitCallback() {
      @Override
      public void onCoreInitFinished() {
        Log.e(TAG, "TBS内核初始化完毕");
      }

      @Override
      public void onViewInitFinished(boolean b) {
        initX5Status = b ? X5Status.SUCCESS : X5Status.FAIL;
        handleMessage(handler);

        if (!b) {
          resetQbSdk();
        }

        Log.e(TAG, "TBS内核初始化" + showStatus(b) + " - " + QbSdk.canLoadX5(mContext));
      }
    };

    // X5内核初始化
    QbSdk.initX5Environment(mContext, onQbSdkPreInitCallback);
    Log.e(TAG, "app是否主动禁用了X5内核: " + QbSdk.getIsSysWebViewForcedByOuter());
  }

  /**
   * 发送消息，主要用于在其他线程时可以在主线程发送消息给Flutter
   * 实现X5内核初始化监听
   */
  private void handleMessage(Handler handler) {
    Message message = handler.obtainMessage();
    handler.sendMessage(message);
  }
  
  /**
   * 显示 状态
   *
   * @param flag
   * @return
   */
  private String showStatus(boolean flag) {
    return flag ? "成功" : "失败";
  }

  /**
   * 重置 QbSdk
   */
  private void resetQbSdk() {
    if (mContext != null && !QbSdk.canLoadX5(mContext)) {
      // 重置
      QbSdk.reset(mContext);
      
      initX5Status = X5Status.INIT;
    } else {
      initX5Status = X5Status.SUCCESS;
    }
  }

  /**
   * 获取X5初始化状态
   *
   * @return
   */
  private int getX5Status() {
    Log.e(TAG, "获取内核加载状态 - " + initX5Status);
    return initX5Status;
  }

  /**
   * 初始化 X5内核
   */
  private void manualInitX5() {
    if (mContext == null) {
      return;
    }

    // 正在初始化 / 内核加载完成 / 下载完成 / 正在下载 / 其他状态，无须重复操作
    // 仅处理 未初始化 / 初始化失败 / 下载失败 的情况
    if (initX5Status == X5Status.INIT) {
      initX5();
    } else if (initX5Status == X5Status.FAIL) {
      if (!QbSdk.canLoadX5(mContext)) {
        initX5();
      }
    } else if (initX5Status == X5Status.DOWNLOAD_FAIL) {
      TbsDownloader.startDownload(mContext);
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    // 清空数据
    channel.setMethodCallHandler(null);
    this.mFlutterPluginBinding = null;
    this.mContext = null;
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    Log.e(TAG, "onAttachedToActivity");

    // App启动时，开始初始化X5内核
    initX5();

    // 因TbsReaderView问题，故在此注册
    mFlutterPluginBinding.getPlatformViewRegistry().registerViewFactory(viewName,
        new FileLocalViewFactory(mFlutterPluginBinding.getBinaryMessenger(), binding.getActivity()));
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
    Log.e(TAG, "onDetachedFromActivityForConfigChanges");
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    Log.e(TAG, "onReattachedToActivityForConfigChanges");
  }

  @Override
  public void onDetachedFromActivity() {
    Log.e(TAG, "onDetachedFromActivity");
  }
}
